# core-image-watcher
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Core Image Watcher functional code."""

import argparse
import configparser
import logging
import os
import subprocess
import time

import kombu
from kombu.common import uuid
from uservice_utils.logging import configure_service_logging

from core_image_watcher import constants


logger = logging.getLogger(__name__)


logging_extra = {
    'solution': constants.SOLUTION_NAME,
    'service': constants.SERVICE_NAME,
    'hostname': constants.HOSTNAME,
}


def _enqueue(body, amqp_uris):
    """Enqueues a message of `body` to the rabbit queue"""
    extra = logging_extra.copy()
    extra.update(body)
    with kombu.Connection(amqp_uris) as connection:
        logger.info('Triggering request: %s', body, extra=extra)
        try:
            queue = connection.SimpleQueue(
                'core.image.{}'.format(constants.API_VERSION))
            queue.put(body)
            queue.close()
        except Exception as exc:
            logger.error(exc, exc_info=True, extra=extra)
        logger.info('Done!', extra=extra)


def _get_version_string_output(channel, device):
    """Obtains a bytestring of the images info from the core image server"""
    cmd = [
        'ubuntu-device-flash',
        'query',
        '--list-images',
        '--channel={}'.format(channel),
        '--device={}'.format(device),
    ]
    images = b''
    try:
        images = subprocess.check_output(cmd)
    except subprocess.CalledProcessError as e:
        logger.error(e, exc_info=True, extra=logging_extra)
    finally:
        return images


def _get_latest_image_version(channel,
                              device,
                              get_output=_get_version_string_output):
    """Returns largest image version"""
    images = get_output(channel, device).split()
    if (len(images) >= 2):
        return images[-2].decode('utf-8').replace(':', '')

    logger.error('Could not get the image list by u-d-f: %s',
                 images, extra=logging_extra)
    return None


def _cache_version_to_disk(location, latest_version):
    """Caches the version of the image if it's larger than a cached one"""
    try:
        if os.path.exists(location):
            with open(location, 'r') as cache:
                if (int(cache.read()) < int(latest_version)):
                    with open(location, 'w') as f:
                        f.write(latest_version)
                else:
                    latest_version = None
        else:
            with open(location, 'w') as f:
                f.write(latest_version)
    except IOError as e:
        latest_version = None
        logger.error('Writing the latest image info failed: %s',
                     str(e), extra=logging_extra)
    finally:
        return latest_version


def _check_for_new_image(location,
                         channel,
                         device,
                         latest_image_version=_get_latest_image_version,
                         cached_version=_cache_version_to_disk):
    """Check if a new image is present in the core image server"""
    latest_version = latest_image_version(channel, device)
    try:
        ret = cached_version(location, latest_version)
        if not ret:
            # There is no new image
            # Do not progress
            return None
    except Exception as e:
        logger.error(e, exc_info=True, extra=logging_extra)
        return None
    body = {
        'image_name': latest_version,
        'channel': channel,
        'device': device,
        'request_id': uuid(),
    }
    return body


def main():
    parser = argparse.ArgumentParser(
        description='Core image watcher ...')
    parser.add_argument('-c', '--conf', default='core-service.conf',
                        help='Configuration file path')
    args = parser.parse_args()

    # Load configuration options.
    config = configparser.ConfigParser()
    config.read(args.conf)

    log_path = os.path.abspath(
        os.path.join(__file__, '../../logs/core-image-watcher.log'))
    configure_service_logging(
        log_path,
        config['logstash'] if 'logstash' in config else None
    )

    logger.info('Started!', extra=logging_extra)

    amqp_uris = config.get('amqp', 'uris').split()
    location = config.get('image', 'location')
    channel = config.get('image', 'channel')
    device = config.get('image', 'device')
    poll_period = float(config.get('image', 'poll_period'))

    try:
        while True:
            message_body = _check_for_new_image(location,
                                                channel,
                                                device)
            if message_body:
                _enqueue(message_body, amqp_uris)
            time.sleep(poll_period)
    except KeyboardInterrupt:
        print('Bye!')
