# Ubuntu CI Engine
# Copyright 2015 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile
import testtools
from testtools.matchers import (
    Equals,
    MatchesDict,
    NotEquals,
)

from core_image_watcher import (
    _cache_version_to_disk,
    _get_latest_image_version,
    _check_for_new_image,
)


class TestWatchImage(testtools.TestCase):

    def write_to_tempfile(self, version):
        temp_file = tempfile.NamedTemporaryFile('w+', delete=False)
        with temp_file as f:
            f.write(version)
        self.addCleanup(os.remove, temp_file.name)
        return temp_file.name

    def test_cache_version_to_disk(self):
        temp_file_name = self.write_to_tempfile('111')
        return_value = _cache_version_to_disk(temp_file_name, '112')
        self.assertEqual(return_value, '112')

    def test_cache_version_with_different_digits_to_disk(self):
        temp_file_name = self.write_to_tempfile('23')
        return_value = _cache_version_to_disk(temp_file_name, '112')
        self.assertEqual(return_value, '112')

    def test_do_not_cache_the_same_version_to_disk(self):
        temp_file_name = self.write_to_tempfile('112')
        return_value = _cache_version_to_disk(temp_file_name, '112')
        self.assertEqual(return_value, None)

    def test_do_not_proceed(self):
        temp_file_name = self.write_to_tempfile('113')
        return_value = _cache_version_to_disk(temp_file_name, '112')
        self.assertEqual(return_value, None)

    def test_freshly_cache_version_to_disk(self):
        temp_file = tempfile.NamedTemporaryFile('w+', delete=False)
        return_value = _cache_version_to_disk(temp_file.name, '112')
        self.addCleanup(os.remove, temp_file.name)
        self.assertEqual(return_value, '112')

    def test_get_latest_version(self):
        def get_output(channel, device):
            return b"295: description='fake295'\n\
                   296: description='fake296'\n\
                   297: description='fake297'\n\
                   298: description='fake338'\n"
        observed = _get_latest_image_version('fakechannel',
                                             'fakedevice',
                                             get_output)
        self.assertEqual(observed, '298')

    def test_dont_get_latest_version(self):
        def get_output(channel, device):
            return b"100: description='fake100'\n"
        observed = _get_latest_image_version('fakechannel',
                                             'fakedevice',
                                             get_output)
        self.assertEqual(observed, '100')

    def test_check_for_new_image(self):
        def _get_latest_image_version(channel, device):
            return '100'

        def _cache_version_to_disk(location, latest_version):
            return '99'

        body = _check_for_new_image('fakelocation',
                                    'fakechannel',
                                    'fakedevice',
                                    _get_latest_image_version,
                                    _cache_version_to_disk)
        self.assertThat(
            body,
            MatchesDict(
                {
                    'device': Equals('fakedevice'),
                    'image_name': Equals('100'),
                    'channel': Equals('fakechannel'),
                    'request_id': NotEquals(''),
                }
            )
        )

    def test_check_for_no_new_image(self):
        def _get_latest_image_version(channel, device):
            return '100'

        def _cache_version_to_disk(location, latest_version):
            return None

        body = _check_for_new_image('fakelocation',
                                    'fakechannel',
                                    'fakedevice',
                                    _get_latest_image_version,
                                    _cache_version_to_disk)
        self.assertEqual(body, None)
